@@ -1,13 +1,13 @@
 ########################################################################
 # is.marssMODEL function
-# Check that the modelObj object has all the parts it needs
+# Check that the MODELobj object has all the parts it needs
 # data, fixed, free, and X.names
 # and that these have the proper size and form
 # That it has all its attributes and 
 # that the fixed and free matrices fit the specified attributes (internally consistent)
 ########################################################################
-is.marssMODEL <- function(modelObj, method="kem") {
-  if(class(modelObj) != "marssMODEL") stop("Stopped in is.marssMODEL() because object class is not marssMODEL.\n", call.=FALSE)
+is.marssMODEL <- function(MODELobj, method="kem") {
+  if(class(MODELobj) != "marssMODEL") stop("Stopped in is.marssMODEL() because object class is not marssMODEL.\n", call.=FALSE)
   msg = NULL
   
   ###########################
@@ -16,8 +16,8 @@ is.marssMODEL <- function(modelObj, method="kem") {
   
   ## Check for required components
   el = c("data","fixed","free","tinitx","diffuse")
-  if( !all(el %in% names(modelObj)) ) { 
-    msg = c(msg, "Element", el[!(el %in% names(modelObj))], "is missing from the model object.\n")
+  if( !all(el %in% names(MODELobj)) ) { 
+    msg = c(msg, "Element", el[!(el %in% names(MODELobj))], "is missing from the model object.\n")
   }
   if(!is.null(msg)){  #rest of the tests won't work so stop now
     msg=c("\nErrors were caught in is.marssMODEL()\n", msg)
@@ -26,21 +26,21 @@ is.marssMODEL <- function(modelObj, method="kem") {
   
   ## Check that free and fixed are numeric matrices with no NA or Infs
   for(mat in c("fixed","free")){
-    if (!is.list(modelObj[[mat]])) msg = c(msg, paste("modelObj$",mat," must be a list of matrices.\n",sep="")) 
-    for (i in 1:length(modelObj[[mat]])) {
-      if(class(modelObj[[mat]][[i]]) != "array" || length(dim(modelObj[[mat]][[i]]))!=3){ 
-        msg = c(msg, paste("modelObj$",mat,"$",names(modelObj[[mat]])[i]," must be a 3D matrix.\n", sep=""))
+    if (!is.list(MODELobj[[mat]])) msg = c(msg, paste("MODELobj$",mat," must be a list of matrices.\n",sep="")) 
+    for (i in 1:length(MODELobj[[mat]])) {
+      if(class(MODELobj[[mat]][[i]]) != "array" || length(dim(MODELobj[[mat]][[i]]))!=3){ 
+        msg = c(msg, paste("MODELobj$",mat,"$",names(MODELobj[[mat]])[i]," must be a 3D matrix.\n", sep=""))
       }
-      if(mode(modelObj[[mat]][[i]]) != "numeric" || any(is.na(modelObj[[mat]][[i]])) || any(is.infinite(modelObj[[mat]][[i]])) ) 
-        msg = c(msg, paste("modelObj$",mat,"$",names(modelObj[[mat]])[i]," must be numeric, have no NAs, and no Infs.\n", sep=""))  
+      if(mode(MODELobj[[mat]][[i]]) != "numeric" || any(is.na(MODELobj[[mat]][[i]])) || any(is.infinite(MODELobj[[mat]][[i]])) ) 
+        msg = c(msg, paste("MODELobj$",mat,"$",names(MODELobj[[mat]])[i]," must be numeric, have no NAs, and no Infs.\n", sep=""))  
     }
   }
   
-  if( length(dim(modelObj$data)) != 2)
+  if( length(dim(MODELobj$data)) != 2)
     msg = c(msg, "Data is not a 2D matrix.\n")
   ## check for T=1
-  if( !is.numeric(modelObj$data ) ) msg = c(msg, "Data must be numeric.\n")
-  if( dim(modelObj$data)[2] == 1 ) msg = c(msg, "Data has only one time point.\n")
+  if( !is.numeric(MODELobj$data ) ) msg = c(msg, "Data must be numeric.\n")
+  if( dim(MODELobj$data)[2] == 1 ) msg = c(msg, "Data has only one time point.\n")
   
   if(!is.null(msg)){  #rest of the tests won't work so stop now
     msg=c("\nErrors were caught in is.marssMODEL()\n", msg)
@@ -51,7 +51,7 @@ is.marssMODEL <- function(modelObj, method="kem") {
   # Check that the attributes are complete and consistent
   ###########################
   el = c("form","model.dims","par.names","X.names","Y.names", "equation","obj.elements")
-  attr.names=names(attributes(modelObj))
+  attr.names=names(attributes(MODELobj))
   if( !all(el %in% attr.names) ) { 
     msg = c(msg, "Element", el[!(el %in% attr.names)], "is missing from the attributes of the model object.\n")
   }
@@ -61,14 +61,14 @@ is.marssMODEL <- function(modelObj, method="kem") {
   }  
   el=c("par.names","form","X.names","Y.names","equation")
   for(elem in el){
-    fattr=attr(modelObj,elem)
+    fattr=attr(MODELobj,elem)
     if(!is.vector(fattr) | !is.character(fattr)){
       msg=c("The ", elem, " attribute of the marssMODEL object needs to be a character vector.\nErrors were caught in is.marssMODEL()\n", msg, sep="")
       return(msg)   #the rest of the tests will hang so stop now 
     }
   }
   
-  par.names=attr(modelObj,"par.names")
+  par.names=attr(MODELobj,"par.names")
   if(any(duplicated(par.names)))
     msg=c(msg, "par.names attribute of the model object has duplicated names.\n")
   ###########################
@@ -77,7 +77,7 @@ is.marssMODEL <- function(modelObj, method="kem") {
   ###########################
   el = c("fixed","free")
   for(elem in el){
-    fnames=names(modelObj[[elem]])
+    fnames=names(MODELobj[[elem]])
     if( !all(par.names %in% fnames ) ) { 
       msg = c(msg, "Element ", par.names[!(par.names %in% fnames)], " is missing from the ", elem," element of the model object.\n", sep="")
     }
@@ -89,7 +89,7 @@ is.marssMODEL <- function(modelObj, method="kem") {
   ###########################
   # Check that model dims have all the par.names
   ###########################
-  model.dims=attr(modelObj,"model.dims")
+  model.dims=attr(MODELobj,"model.dims")
   #the info in model.dims should be par.names and the extras
   model.dim.names=c(par.names,"data","x","y","w","v")
   fnames=names(model.dims)
@@ -110,16 +110,16 @@ is.marssMODEL <- function(modelObj, method="kem") {
   # Check that length of X.names matches first dim of model.dims$X
   # Check that length of Y.names matches first dim of model.dims$Y
   ###########################
-  if(length(attr(modelObj,"X.names"))!=attr(modelObj,"model.dims")$x[1])
+  if(length(attr(MODELobj,"X.names"))!=attr(MODELobj,"model.dims")$x[1])
     msg="The length of the X.names attribute of model object must equal the first element of the model.dims attribute x element.\n"
-  if(length(attr(modelObj,"Y.names"))!=attr(modelObj,"model.dims")$y[1])
+  if(length(attr(MODELobj,"Y.names"))!=attr(MODELobj,"model.dims")$y[1])
     msg=c(msg,"The length of the Y.names attribute of model object must equal the first element of the model.dims attribute y element.\n")
   ###########################
   # Check that 2nd dim of model.dims$x and model.dims$y equals the 2nd dim of data
   ###########################
-  TT=dim(modelObj$data)[2]
-  if(attr(modelObj,"model.dims")$y[2]!=TT | attr(modelObj,"model.dims")$x[2]!=TT |
-       attr(modelObj,"model.dims")$w[2]!=TT | attr(modelObj,"model.dims")$v[2]!=TT )
+  TT=dim(MODELobj$data)[2]
+  if(attr(MODELobj,"model.dims")$y[2]!=TT | attr(MODELobj,"model.dims")$x[2]!=TT |
+       attr(MODELobj,"model.dims")$w[2]!=TT | attr(MODELobj,"model.dims")$v[2]!=TT )
     msg=c(msg,"The 2nd element of the model.dims attribute for x, y, w, and v must equal the number of time points in the data.\n")
 
   if(!is.null(msg)){  #rest of the tests won't work so stop now
@@ -131,7 +131,7 @@ is.marssMODEL <- function(modelObj, method="kem") {
   # Check that fixed and free are 3D
   ###########################
   dim.fixed = dim.free =  NULL
-  free=modelObj$free; fixed=modelObj$fixed
+  free=MODELobj$free; fixed=MODELobj$fixed
   for (elem in par.names) {
     dim.fixed.flag = dim.free.flag = FALSE
     if(length(dim(free[[elem]]))!=3){ dim.free.flag = TRUE } #3-dimensions
@@ -213,22 +213,22 @@ is.marssMODEL <- function(modelObj, method="kem") {
   # Check data and missing values consistency if data present
   # as.numeric(NA) is the missing value
   ###########################
-  if(!is.numeric(modelObj$data)) msg = paste(msg, "Data must be numeric. \n")
+  if(!is.numeric(MODELobj$data)) msg = paste(msg, "Data must be numeric. \n")
   for( bad.val in c(NA, NaN, Inf, -Inf)){
-    if(!identical(bad.val, as.numeric(NA)) && ( bad.val %in% modelObj$data ) ){  
+    if(!identical(bad.val, as.numeric(NA)) && ( bad.val %in% MODELobj$data ) ){  
       msg = c(msg, paste("Data cannot have ", bad.val,". \n",sep="")) }
   }
 
   ###########################
   # Y.names against rownames of data.  They should be identical; otherwise something got scrambled
   ###########################
-  if(!identical(attr(modelObj,"Y.names"),rownames(modelObj$data)) ){  
+  if(!identical(attr(MODELobj,"Y.names"),rownames(MODELobj$data)) ){  
     msg = c(msg, paste("The rownames of the data and the attribute Y.names don't match.\n",sep="")) }
   
   ###########################
   # Check tinitx; must be 0 or 1
   ###########################
-  if( !(modelObj$tinitx %in% c(0,1)) ){ msg = c(msg, "tinitx (t associated with initial x) must be 0 or 1.\n") }
+  if( !(MODELobj$tinitx %in% c(0,1)) ){ msg = c(msg, "tinitx (t associated with initial x) must be 0 or 1.\n") }
   
   if(!is.null(msg)){  #next test won't work so stop now
     msg=c("\nErrors were caught in is.marssMODEL()\n", msg)
@@ -239,12 +239,12 @@ is.marssMODEL <- function(modelObj, method="kem") {
   # Last Check that fixed, free, par.names are complete and consistent
   # This is form dependent so the MARSS.form file needs to include a is.marssMODEL_form() function
   ###########################
-  form=attr(modelObj, "form")
+  form=attr(MODELobj, "form")
   is.marssMODEL.fun = paste("is.marssMODEL_", form[1], sep="")
   tmp=try(exists(is.marssMODEL.fun, mode="function"),silent=TRUE)
   if(isTRUE(tmp)){
     #the is.marssMODEL_form function runs tests and then returns msgs
-    msg=eval(call(is.marssMODEL.fun, modelObj, method=method))
+    msg=eval(call(is.marssMODEL.fun, MODELobj, method=method))
   }else{ 
     msg=c(msg, paste("No is.marssMODEL_", form[1], " is available to test the model object.\n", sep=""))
   }
@@ -252,7 +252,7 @@ is.marssMODEL <- function(modelObj, method="kem") {
   ###########################
   # Check diffuse; must be TRUE or FALSE
   ###########################
-  if( !(modelObj$diffuse %in% c(FALSE, TRUE)) ){ msg = c(msg, "diffuse must be TRUE or FALSE.\n") }
+  if( !(MODELobj$diffuse %in% c(FALSE, TRUE)) ){ msg = c(msg, "diffuse must be TRUE or FALSE.\n") }
   
   if(length(msg) == 0){ return(TRUE)
   }else{
